// Sound activated by an HC-SR04 proximity sensor. 

// Wiring for proximity sensor as follows:
//
// trig  -->  A0
// echo  -->  A1
// gnd   -->  gnd
// vcc   -->  +5V
//
// You can use any of the digital or analog pins for the trigger and echo, but this was limited when using the adafruit wave shield
// and the easiest ones to physically access were A0 and A1.
// 
// I have also included a simple funciton to randomly select a new wave file when the current one has finished playing. The filenames
// are contained in array. Note: if you modifythe array please DON'T forget to put commas between each character string, including
// the ones at the end of each line unless you want a hard-to-find bug.
// 
// Drop me a line if you get stuck at: n.d.hollinworth@reading.ac.uk
 
#include <FatReader.h>
#include <SdReader.h>
#include <avr/pgmspace.h>
#include "WaveUtil.h"
#include "WaveHC.h"

SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the filesystem on the card
FatReader f;      // This holds the information for the file we're play

WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time

const int trigPin = A0;  // A0 on the UNO
const int echoPin = A1;  // A1 on the UNO
const int MIN_DIST = 20;
char* FILENAME = "HOOVES.WAV";
char* FILES [] = {"BARORGAN.WAV", "BEATBOWL.WAV", "BELLRING.WAV", "CHIMES12.WAV", "CHOPPING.WAV", "CREAKING.WAV",
                  "CUPFILL.WAV", "DINNER.WAV", "DOORBOLT.WAV", "DOOROPEN.WAV", "FIRE.WAV", "FOOTSTEP.WAV", "HOOVES.WAV",
                  "KITBELLS.WAV", "KITCHEN.WAV", "LOCKCLCK.WAV", "PASTRY.WAV", "RINSECUP.WAV", "SHEILA.WAV", "STIRRING.WAV",
                  "TAP.WAV", "WALKLAND.WAV"};

// this handy function will return the number of bytes currently free in RAM, great for debugging!   
int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 

void sdErrorCheck(void)
{
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

void setup() {
  // initialize serial communication:
  Serial.begin(9600);
  
  putstring("Free RAM: ");        // This can help with debugging, running out of RAM is bad
  Serial.println(freeRam());      // if this is under 150 bytes it may spell trouble!
  
  // Set the output pins for the DAC control. This pins are defined in the library
  setupDACPins();
  
  initSDCard();
  
  // Load wave file. We only need to do this once as we're playing asingle sound file
  // Alternatively, load wave files arbitrarily.
  //loadWaveFile(FILENAME);
  loadRandomFile();
}

void loop()
{
  // establish variables for duration of the ping, 
  // and the distance result in inches and centimeters:
  long duration, inches, cm;

  // The sensor is triggered by a HIGH pulse of 10 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  pinMode(trigPin, OUTPUT);
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  // Read the signal from the sensor: a HIGH pulse whose
  // duration is the time (in microseconds) from the sending
  // of the ping to the reception of its echo off of an object.
  pinMode(echoPin, INPUT);
  duration = pulseIn(echoPin, HIGH);

  // convert the time into a distance
  //inches = microsecondsToInches(duration);
  cm = microsecondsToCentimeters(duration);
  
  // If the wave file has already been started (isplaying = 1) and distance is greater than the threshold distance
  // then pause the playback. If the distance is greater than the threshold and wave file has finished, then restart
  if(cm > MIN_DIST + 10) {
    if (wave.isplaying) {
      wave.pause();
    } else {
      loadRandomFile();
      //wave.seek(0); 
    }
  }
  
  if (cm <= MIN_DIST) {
    if (!wave.isplaying) {
      wave.play();
    } else {
      wave.resume(); 
    }
  }
  
  
  //Serial.println(cm);
  delay(10);
}

void loadWaveFile(char * name) {
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file "); Serial.print(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }  
}

// Randomly select and load one of the wav files. This is called when each file has finished playing
void loadRandomFile() {
  
  long randNum = random(22);
  loadWaveFile(FILES[randNum]);
  
}

void setupDACPins() {
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);  
}

void initSDCard() {
  if (!card.init()) {         //play with 8 MHz spi (default faster!)  
    putstring_nl("Card init. failed!");  // Something went wrong, lets print out why
    sdErrorCheck();
    while(1);                            // then 'halt' - do nothing!
  }
  
  // enable optimize read - some cards may timeout. Disable if you're having problems
  card.partialBlockRead(true);
  
  // Now we will look for a FAT partition!
  uint8_t part;
  for (part = 0; part < 5; part++) {     // we have up to 5 slots to look in
    if (vol.init(card, part)) 
      break;                             // we found one, lets bail
  }
  
  if (part == 5) {                       // if we ended up not finding one  :(
    putstring_nl("No valid FAT partition!");
    sdErrorCheck();      // Something went wrong, lets print out why
    while(1);                            // then 'halt' - do nothing!
  }
  
  // Lets tell the user about what we found
  putstring("Using partition ");
  Serial.print(part, DEC);
  putstring(", type is FAT");
  Serial.println(vol.fatType(),DEC);     // FAT16 or FAT32?
  
  // Try to open the root directory
  if (!root.openRoot(vol)) {
    putstring_nl("Can't open root dir!"); // Something went wrong,
    while(1);                             // then 'halt' - do nothing!
  }  
}

long microsecondsToInches(long microseconds)
{
  // According to Parallax's datasheet for the PING))), there are
  // 73.746 microseconds per inch (i.e. sound travels at 1130 feet per
  // second).  This gives the distance travelled by the ping, outbound
  // and return, so we divide by 2 to get the distance of the obstacle.
  // See: http://www.parallax.com/dl/docs/prod/acc/28015-PING-v1.3.pdf
  return microseconds / 74 / 2;
}

long microsecondsToCentimeters(long microseconds)
{
  // The speed of sound is 340 m/s or 29 microseconds per centimeter.
  // The ping travels out and back, so to find the distance of the
  // object we take half of the distance travelled.
  return microseconds / 29 / 2;
}

// Plays a full file from beginning to end with no pause.
void playcomplete(char *name) {
  // call our helper to find and play this name
  playfile(name);
  while (wave.isplaying) {
  // do nothing while its playing
  }
  // now its done playing
}

void playfile(char *name) {
  // see if the wave object is currently doing something
  if (wave.isplaying) {// already playing something, so stop it!
    wave.stop(); // stop it
  }
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file "); Serial.print(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }
  
  // ok time to play! start playback
  wave.play();
}
